@@ -78,6 +78,15 @@ totem_interface_error_dialog (const char *title, const char *reason,
 	return error_dialog;
 }
 
+/**
+ * totem_interface_error:
+ * @title: the error title
+ * @reason: the error reason (secondary text)
+ * @parent: the error dialogue's parent #GtkWindow
+ *
+ * Display a modal error dialogue with @title as its primary error text, and @reason
+ * as its secondary text.
+ **/
 void
 totem_interface_error (const char *title, const char *reason,
 		GtkWindow *parent)
@@ -92,6 +101,15 @@ totem_interface_error (const char *title, const char *reason,
 	gtk_window_present (GTK_WINDOW (error_dialog));
 }
 
+/**
+ * totem_interface_error_blocking:
+ * @title: the error title
+ * @reason: the error reason (secondary text)
+ * @parent: the error dialogue's parent #GtkWindow
+ *
+ * Display a modal error dialogue like totem_interface_error() which blocks until the user has
+ * dismissed it.
+ **/
 void
 totem_interface_error_blocking (const char *title, const char *reason,
 		GtkWindow *parent)
@@ -147,7 +165,7 @@ link_button_clicked_cb (GtkWidget *widget, Totem *totem)
  * @parent: the error dialogue's parent #GtkWindow
  * @totem: a #TotemObject
  *
- * Display a modal error dialogue like totem_interface_error_dialog(),
+ * Display a modal error dialogue like totem_interface_error(),
  * but add a button which will open @uri in a browser window.
  **/
 void
@@ -166,6 +184,7 @@ totem_interface_error_with_link (const char *title, const char *reason,
 	hbox = gtk_hbox_new (TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), link_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (error_dialog)->vbox), hbox, TRUE, FALSE, 0); 
+	gtk_widget_show_all (hbox);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog), GTK_RESPONSE_OK);
 
@@ -354,71 +373,3 @@ totem_interface_get_license (void)
 			  NULL);
 }
 
-void
-totem_interface_boldify_label (GtkBuilder *builder, const char *name)
-{
-	GtkLabel *widget;
-	gchar *str_final;
-
-	widget = GTK_LABEL (gtk_builder_get_object (builder, name));
-
-	if (widget == NULL) {
-		g_warning ("widget '%s' not found", name);
-		return;
-	}
-
-	/* this way is probably better, but for some reason doesn't work with
-	 * labels with mnemonics.
-
-	static PangoAttrList *pattrlist = NULL;
-
-	if (pattrlist == NULL) {
-		PangoAttribute *attr;
-
-		pattrlist = pango_attr_list_new ();
-		attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-		attr->start_index = 0;
-		attr->end_index = G_MAXINT;
-		pango_attr_list_insert (pattrlist, attr);
-	}
-	gtk_label_set_attributes (widget, pattrlist);*/
-
-	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (widget));
-	gtk_label_set_markup_with_mnemonic (widget, str_final);
-	g_free (str_final);
-}
-
-void
-totem_interface_italicise_label (GtkBuilder *builder, const char *name)
-{
-	GtkLabel *widget;
-	gchar *str_final;
-
-	widget = GTK_LABEL (gtk_builder_get_object (builder, name));
-
-	if (widget == NULL) {
-		g_warning ("widget '%s' not found", name);
-		return;
-	}
-
-	/* this way is probably better, but for some reason doesn't work with
-	 * labels with mnemonics.
-
-	static PangoAttrList *pattrlist = NULL;
-
-	if (pattrlist == NULL) {
-		PangoAttribute *attr;
-
-		pattrlist = pango_attr_list_new ();
-		attr = pango_attr_style_new (PANGO_STYLE_ITALIC);
-		attr->start_index = 0;
-		attr->end_index = G_MAXINT;
-		pango_attr_list_insert (pattrlist, attr);
-	}
-	gtk_label_set_attributes (widget, pattrlist);*/
-
-	str_final = g_strdup_printf ("<i>%s</i>", gtk_label_get_label (widget));
-	gtk_label_set_markup_with_mnemonic (widget, str_final);
-	g_free (str_final);
-}
-
