@@ -162,10 +162,7 @@ struct TotemObject {
 	TotemStates state;
 	TotemOpenLocation *open_location;
 	gboolean remember_position;
-
-	/* Stored source_window of GdkDragContext from a video drag,
-	 * used to detect when the video is dropped on itself. */
-	GdkWindow *video_drag_source_window;
+	gboolean disable_kbd_shortcuts;
 };
 
 GtkWidget *totem_volume_create (void);
